from ctypes import (c_int, CDLL, c_char_p, c_wchar_p, c_void_p,
        c_ushort, c_ubyte, byref)
from ctypes.util import find_library
class Blink1LibraryError(Exception):
    pass

libname = find_library("blink1")
if not libname:
    raise Blink1LibraryError('blink(1) library not found')

libblink1 = CDLL(libname)

blink1_open = libblink1.blink1_open
blink1_open.restype = c_void_p

blink1_close = libblink1.blink1_close
blink1_close.argtypes = [c_void_p]

openByPath = libblink1.blink1_openByPath
openByPath.restype = c_void_p
openByPath.argtypes = [c_char_p]

def open_by_path(path):
    return openByPath(path)

openBySerial = libblink1.blink1_openBySerial
openBySerial.restype = c_void_p
openBySerial.argtypes = [c_wchar_p]

def open_by_serial(serial):
    return openBySerial(serial)

openById = libblink1.blink1_openById
openById.restype = c_void_p
openById.argtypes = [c_int]

def open_by_id(id):
    return openById(id)

fadeToRGB = libblink1.blink1_fadeToRGB
fadeToRGB.argtypes = [c_void_p, c_ushort, c_ubyte, c_ubyte, c_ubyte]

def fade_to_rgb(device, time, r, g, b):
    time = c_ushort(time)
    r = c_ubyte(r)
    g = c_ubyte(g)
    b = c_ubyte(b)
    return fadeToRGB(device, time, r, g, b)

setRGB = libblink1.blink1_setRGB
setRGB.argtypes = [c_void_p, c_ubyte, c_ubyte, c_ubyte]

def set_rgb(device, r, g, b):
    r = c_ubyte(r)
    g = c_ubyte(g)
    b = c_ubyte(b)
    return setRGB(device, r, g, b)

_play = libblink1.blink1_play
_play.argtypes = [c_void_p, c_ubyte, c_ubyte]

def play(device, play, pos):
    play = c_ubyte(play)
    pos = c_ubyte(pos)
    _play(device, play, pos)

writePatternLine = libblink1.blink1_writePatternLine
writePatternLine.argtypes = [c_void_p, c_ushort, c_ubyte, c_ubyte, c_ubyte, c_ubyte]

def write_pattern_line(device, time, r, g, b, pos):
    time = c_ushort(time)
    r = c_ubyte(r)
    g = c_ubyte(g)
    b = c_ubyte(b)
    pos = c_ubyte(pos)
    writePatternLine(device, time, r, g, b, pos)

readPatternLine = libblink1.blink1_readPatternLine
readPatternLine.argtypes = [c_void_p, c_void_p, c_void_p, c_void_p, c_void_p, c_ubyte]

def read_pattern_line(device, pos):
    time = c_ushort()
    r = c_ubyte()
    g = c_ubyte()
    b = c_ubyte()
    pos = c_ubyte(pos)
    readPatternLine(device, byref(time), byref(r), byref(g), byref(b),
            pos)
    return int(time.value), int(r.value), int(g.value), int(b.value)



from setuptools import setup

setup(
    name='blink1py',
    url='https://github.com/TronPaul/blink1py',
    version='0.1.0',
    author='Mark McGuire',
    author_email='mark.b.mcg@gmail.com',
    packages=['blink1py'],
    license='Apache2',
    description='blink(1) library wrapper',
    long_description=open('README.rst').read(),
)



import blink1_raw as b1raw

class Blink1NotFoundError(Exception):
    pass

def open_blink1(id=None, serial=None, path=None):
    if id is not None:
        blink1 = b1raw.open_by_id(id)
    elif serial is not None:
        blink1 = b1raw.open_by_serial(serial)
    elif path is not None:
        blink1 = b1raw.open_by_path(path)
    else:
        blink1 = b1raw.blink1_open()
    if not blink1:
        raise Blink1NotFoundError(
            'blink(1) library could not find a blink(1) device')
    return Blink1(blink1)

def close_blink1(blink1):
    b1raw.blink1_close(blink1)

class Blink1(object):
    def __init__(self, device):
        self._device = device
        self.__closed = False

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def __del__(self):
        try:
            self.close()
        except:
            pass

    def close(self):
        if not self.__closed:
            try:
                close_blink1(self._device)
            finally:
                self.__closed = True

    def play(self, pos=0):
        b1raw.play(self._device, 1, pos)

    def stop(self, pos=0):
        b1raw.play(self._device, 0, pos)

    def set_pattern(self, pos, r=0, g=0, b=0, time=0):
        b1raw.write_pattern_line(self._device, time, r, g, b, pos)

    def read_pattern(self, pos):
        time, r, g, b = b1raw.read_pattern_line(self._device, pos)
        return r, g, b, time

    def off(self):
        self.set_rgb(0, 0, 0)

    def on(self):
        self.set_rgb(255, 255, 255)

    def set_rgb(self, r=0, g=0, b=0):
        self.fade_rgb(r, g, b)

    def fade_rgb(self, r=0, g=0, b=0, time=0):
        if not time:
            b1raw.set_rgb(self._device, r, g, b)
        else:
            b1raw.fade_to_rgb(self._device, time, r, g, b)
